isVariantEnd property
bool
get
isVariantEnd
Checks if the game is over due to a special variant end condition.
Implementation
bool get isVariantEnd;
Checks if the game is over due to a special variant end condition.
bool get isVariantEnd;