kingAttackers method
Attacks that a king on square would have to deal with.
Implementation
SquareSet kingAttackers(Square square, Side attacker, {SquareSet? occupied}) {
return board.attacksTo(square, attacker, occupied: occupied);
}
Attacks that a king on square would have to deal with.
SquareSet kingAttackers(Square square, Side attacker, {SquareSet? occupied}) {
return board.attacksTo(square, attacker, occupied: occupied);
}