promoted property
All squares occupied by pieces known to be promoted.
This information is relevant in chess variants like Crazyhouse.
Implementation
final SquareSet promoted;
All squares occupied by pieces known to be promoted.
This information is relevant in chess variants like Crazyhouse.
final SquareSet promoted;